<?php

namespace Teknisa\Model\Teknisa\Repositories;

class OmAtualizacaoEstrutura extends \Doctrine\ORM\EntityRepository {

    public function findStructureUpdate($orgId, $conditions, $order, $limit, $offset){
        $query = <<<SQL
		SELECT ATU.ID, ATU.DSATUESTRUTURA, ATU.DSDIRLOG, ATU.TICKET_ID,
               ATU.CDOPERINCLUSAO, ATU.DTINCLUSAO, ATU.SCRIPT, ATU.SCRIPT_SUC, ATU.SCRIPT_ERR,
               EST.NMOPERADOR, EST.ORGANIZATION_IDS, ATU.IDRESULTSCRIPT,
               CASE WHEN LENGTH(ATU.SCRIPT) > 100
                    THEN SUBSTR(ATU.SCRIPT, 0, 100) || '...'
                    ELSE ATU.SCRIPT
                END AS SCRIPT_GRID
          FROM (SELECT OAE.ID, OPE.NMOPERADOR, OAE.TICKET_ID,
                       RTRIM(XMLAGG(XMLELEMENT(E, OAO.NRORG_ID, ',')
                                    .EXTRACT('//text()')
                                    ORDER BY OAO.NRORG_ID
                                   ).GetClobVal(), ','
                            ) as ORGANIZATION_IDS
                  FROM OM_ATUALIZACAO_ESTRUTURA OAE
                  JOIN OM_ATUALIZACAO_ESTRUTURA_ORG OAO ON OAO.NRATUESTRUTID = OAE.ID
                  JOIN OPERADOR OPE ON OPE.CDOPERADOR = OAE.CDOPERINCLUSAO
                 WHERE OAE.NRORG = :NRORG
                       $conditions
                 GROUP BY OAE.ID, OPE.NMOPERADOR, OAE.TICKET_ID) EST
          JOIN OM_ATUALIZACAO_ESTRUTURA ATU ON ATU.ID = EST.ID
          $order
SQL;
        $param = array( 'NRORG' => $orgId );
        $query = \OrganizationsManagement\Util\Utilities::getPaginatedQuery($query, $limit, $offset);
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
            $register['ORGANIZATION_IDS'] = \OrganizationsManagement\Util\Utilities::setSelectMultiInQuery($register['ORGANIZATION_IDS']);
        }
        return $registers;
    }
    
    public function findStructureUpdateByScriptId($orgId, $scriptId, $conditions, $order, $limit, $offset){
        $query = <<<SQL
		SELECT ATU.ID, EST.STRUCT_ID, ATU.DSATUESTRUTURA, EST.DSDIRLOG,
               ATU.CDOPERINCLUSAO, ATU.DTINCLUSAO, ATU.SCRIPT, ATU.SCRIPT_SUC, ATU.SCRIPT_ERR,
               EST.NMOPERADOR, EST.ORGANIZATION_ID, EST.IDRESULTSCRIPT,
               CASE WHEN LENGTH(ATU.SCRIPT) > 100
                    THEN SUBSTR(ATU.SCRIPT, 0, 100) || '...'
                    ELSE ATU.SCRIPT
                END AS SCRIPT_GRID
          FROM (SELECT OAE.ID, OPE.NMOPERADOR, OAO.ID AS STRUCT_ID,
                       OAO.NRORG_ID AS ORGANIZATION_ID, OAO.IDRESULTSCRIPT, OAO.DSDIRLOG
                  FROM OM_ATUALIZACAO_ESTRUTURA OAE
                  JOIN OM_ATUALIZACAO_ESTRUTURA_ORG OAO ON OAO.NRATUESTRUTID = OAE.ID
                  JOIN OPERADOR OPE ON OPE.CDOPERADOR = OAE.CDOPERINCLUSAO
                 WHERE OAE.NRORG = :NRORG
                 AND OAE.ID = :ID
                 $conditions
                 ) EST
          JOIN OM_ATUALIZACAO_ESTRUTURA ATU ON ATU.ID = EST.ID
SQL;
        $param = array( 'NRORG' =>  $orgId,
                        'ID'    =>  $scriptId
                      );
        $query = \OrganizationsManagement\Util\Utilities::getPaginatedQuery($query, $limit, $offset);
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
         foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findErrorOrPartialOrganizations($id, $result){
        $query = <<<SQL
        SELECT OAO.NRORG_ID
          FROM OM_ATUALIZACAO_ESTRUTURA OAE
          LEFT JOIN OM_ATUALIZACAO_ESTRUTURA_ORG OAO ON OAE.ID = OAO.NRATUESTRUTID
          WHERE OAO.IDRESULTSCRIPT = :RESULT
          AND OAE.ID = :ID
SQL;
        $param = array(
            'ID' => $id,
            'RESULT' => $result
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);

        return $result;
    }
}